package com.persagy.dmp.rwd.edit.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.date.DateUtils;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.config.web.UserUtils;
import com.persagy.dmp.rwd.edit.entity.ClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.FuncidDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.QRwdeditVersion;
import com.persagy.dmp.rwd.edit.entity.RwdeditVersion;
import com.persagy.dmp.rwd.edit.repository.RwdeditVersionRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import javax.transaction.Transactional;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;

@Service
public class RwdeditVersionService {

    @Autowired
    private CriteriaUtils criteriaUtils;

    @Autowired
    private RwdeditVersionRepository repository;

    @Autowired
    private ClassDefChangeRecordService classDefChangeRecordService;

    @Autowired
    private FuncidDefChangeRecordService funcidDefChangeRecordService;

    private List<BooleanExpression> parse(ObjectNode criteria) {
        List<BooleanExpression> list = new LinkedList<>();
        QRwdeditVersion qt = QRwdeditVersion.rwdeditVersion;
        list.addAll(CriteriaUtils.parse(qt.version, criteria.get("version")));
        list.addAll(CriteriaUtils.parse(qt.createTime, criteria.get("createTime")));
        list.addAll(CriteriaUtils.parse(qt.createUser, criteria.get("createUser")));
        list.addAll(CriteriaUtils.parse(qt.state, criteria.get("state")));
        return list;
    }

    public PagedResponse<RwdeditVersion> query(JacksonCriteria criteria) {
        return criteriaUtils.query(QRwdeditVersion.rwdeditVersion, this::parse, criteria);
    }

    @Transactional
    public MapResponse create(RwdeditVersion param) {
        MapResponse response = new MapResponse();
        // TODO
        String version = param.getVersion();
        if (version == null) {
            response.setFail("版本编号不可为空!");
            return response;
        }
        long count = repository.count(QRwdeditVersion.rwdeditVersion.version.eq(version));
        if (count > 0) {
            response.setFail("版本编号已存在!");
            return response;
        }

        RwdeditVersion entity = new RwdeditVersion();
        entity.setVersion(version);
        entity.setState(1);
        entity.setCreateTime(new Date());
        entity.setCreateUser(UserUtils.currentUserId() + "");
        repository.save(entity);
        response.add("id", entity.getVersion());
        return response;
    }

    @Transactional
    public MapResponse update(RwdeditVersion param) {
        MapResponse response = new MapResponse();
        String version = param.getVersion();
        Integer state = param.getState();
        if (version == null) {
            response.setFail("version is required");
            return response;
        }
        if (state == null) {
            response.setFail("state is required");
            return response;
        }
        Optional<RwdeditVersion> data = repository.findById(version);
        if (!data.isPresent()) {
            response.setFail("version not exists");
            return response;
        }
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("version", version);
        //修改类型变更记录状态
        List<ClassDefChangeRecord> classDefList = classDefChangeRecordService.query(criteria).getData();
        if (classDefList != null) {
            for (ClassDefChangeRecord classDefChangeRecord : classDefList) {
                classDefChangeRecord.setState(state);
            }
            classDefChangeRecordService.update(classDefList);
        }
        //修改信息点变更记录状态
        List<FuncidDefChangeRecord> funcidDefList = funcidDefChangeRecordService.query(criteria).getData();
        if (funcidDefList != null) {
            for (FuncidDefChangeRecord funcidDefChangeRecord : funcidDefList) {
                funcidDefChangeRecord.setState(state);
            }
            funcidDefChangeRecordService.update(funcidDefList);
        }
        RwdeditVersion rwdeditVersion = data.get();
        rwdeditVersion.setState(state);
        repository.save(rwdeditVersion);
        return response;
    }

    @Transactional
    public MapResponse delete(RwdeditVersion param) {
        MapResponse response = new MapResponse();
        String version = param.getVersion();
        Integer state = param.getState();
        if (version == null) {
            response.setFail("version is required");
            return response;
        }
        Optional<RwdeditVersion> data = repository.findById(version);
        if (!data.isPresent()) {
            response.setFail("version not exists");
            return response;
        }
        RwdeditVersion rwdeditVersion = data.get();
        if (state == 1) {
            repository.delete(rwdeditVersion);
        } else {
            JacksonCriteria criteria = JacksonCriteria.newInstance();
            criteria.add("version", rwdeditVersion.getVersion());
            PagedResponse<ClassDefChangeRecord> classDefResponse = classDefChangeRecordService.query(criteria);
            PagedResponse<FuncidDefChangeRecord> funcidDefResponse = funcidDefChangeRecordService.query(criteria);
            if (classDefResponse.getCount() == 0 && funcidDefResponse.getCount() == 0) {
                repository.delete(rwdeditVersion);
            } else {
                response.setFail("该版本有变更记录不可直接删除");
            }
        }
        return response;
    }

    @Transactional
    public void download(HttpServletResponse resp, String version) {
        String fileName = DateUtils.format(new Date()) + ".sql";
        List<String> sqlList = new ArrayList<>();
        sqlList.addAll(classDefChangeRecordService.getSqlList(version));
        sqlList.addAll(funcidDefChangeRecordService.getSqlList(version));

        OutputStream out = null;
        try {
            InputStream stream = getStream(sqlList);
            resp.reset();// 清空输出流
            String resultFileName = URLEncoder.encode(fileName, "UTF-8");
            resp.setCharacterEncoding("UTF-8");
            // 设定输出文件头
            resp.setHeader("Content-disposition", "attachment; filename=" + resultFileName);
            // 定义输出类型
            resp.setContentType("application/txt");
            out = resp.getOutputStream();
            byte[] buff = new byte[4096];
            int size = 0;
            while ((size = stream.read(buff)) != -1) {
                out.write(buff, 0, size);
            }
            out.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public InputStream getStream(List<String> list) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream out = new DataOutputStream(baos);
        for (String sql : list) {
            out.writeBytes(sql + "\r\n");
        }
        byte[] bytes = baos.toByteArray();
        ByteArrayInputStream tInputStringStream = new ByteArrayInputStream(bytes);
        baos.close();
        out.close();
        return tInputStringStream;
    }
}

